!********************************************************
module SLHA_manip
!********************************************************
 use string_manip

 implicit none
 private 
 public :: readSLHAfile,writeSLHAfile,finishwithSLHA, &
    &      get_mass,get_totaldecaywidth,get_twobodybranchingratio, &
    &      get_SPhenocrosssection,get_SPhenocrosssectionCMenergy,&
    &      get_threebodybranchingratio,get_modsel,&
    &      get_HiggsCouplingsBosons,get_HiggsCouplingsFermions, &
    &      writeSLHAfile_exceptblock


 type array_of_strings      
  character(len=200) :: line       
 end type

 type(array_of_strings), allocatable :: contents_of_input_file(:)
 type(array_of_strings), allocatable :: edited_contents_of_input_file(:)
 integer :: n_input_lines

 contains

 
 !********************************************************
 subroutine readSLHAfile(fileid)
 !reads in contents of SLHA file, and stores it in 'contents_of_input_file'
 ! strips off comments and tabs and
 ! stores it in 'edited_contents_of_input_file' line-by-line 
 !********************************************************
  implicit none 
  integer,intent(in) :: fileid
  integer :: i,a
  integer :: stringlength
  character(len=200) :: temp_line
  
  !finds number of lines to read in
  n_input_lines=getSLHAfilelength(fileid)
  allocate(contents_of_input_file(n_input_lines)) 
  allocate(edited_contents_of_input_file(n_input_lines))   

  !reads each line in to 'contents_of_input_file'
  do i=1,n_input_lines
   read(fileid,'(a)')contents_of_input_file(i)%line
  enddo

  !this gets rid of the comments and tabs and puts the result into 'edited_contents_of_input_file'
  do i=1,n_input_lines
   temp_line=contents_of_input_file(i)%line
   stringlength=len(temp_line)
   temp_line=strip_off_comment(temp_line,stringlength)  

   a=1
   do while (a.gt.0)  !replace all tab characters with spaces
    a=INDEX(temp_line,'	')
    temp_line=temp_line(:a-1)//'   '//temp_line(a+1:)
   enddo

   edited_contents_of_input_file(i)%line=temp_line
  enddo 

 end subroutine readSLHAfile

 !********************************************************
 subroutine writeSLHAfile(fileid)
 ! writes the inputted data from an SLHA file to the file with
 ! the filehandle=fileid. Includes original comments and tabs
 !********************************************************
  implicit none 
  integer,intent(in) :: fileid
  integer :: i
  
  do i=1,n_input_lines
    write(fileid,'(a)')trim(contents_of_input_file(i)%line)
  enddo
 
 end subroutine writeSLHAfile

 !********************************************************
 subroutine writeSLHAfile_exceptblock(name_of_block,fileid)
 ! writes the inputted data from an SLHA file to the file with
 ! the filehandle=fileid. Includes original comments and tabs
 ! does not write out the block with name 'name_of_block'
 !********************************************************
  implicit none 
  integer,intent(in) :: fileid
  character(len=*),intent(in) ::name_of_block
  integer :: blocklines(2)
  integer :: i
  
  blocklines=line_numbers_of_block(name_of_block)

  if(minval(blocklines).le.0)then !block is not there
   do i=1,n_input_lines
    write(fileid,'(a)')trim(contents_of_input_file(i)%line)
   enddo
  else
   do i=1,blocklines(1)-1
    write(fileid,'(a)')trim(contents_of_input_file(i)%line)
   enddo
   do i=blocklines(2)+1,n_input_lines
    write(fileid,'(a)')trim(contents_of_input_file(i)%line)
   enddo
  endif
 
 end subroutine writeSLHAfile_exceptblock

 !********************************************************
 subroutine finishwithSLHA
 !********************************************************
  implicit none 
   deallocate(contents_of_input_file)
   deallocate(edited_contents_of_input_file)
 end subroutine finishwithSLHA

 !********************************************************
 function get_totaldecaywidth(particlePDGcode)
 !********************************************************
  implicit none
  integer,intent(in) :: particlePDGcode
  integer :: particlePDGcode_fromblock
  integer :: blocklines(2)
  double precision :: get_totaldecaywidth
  character(len=50),allocatable :: col(:)
  character(len=200) :: temp_line
  integer :: stringlength,stringlength2
  integer :: i

  allocate(col(3))
  get_totaldecaywidth=0.0D0

  blocklines=line_numbers_of_decay(particlePDGcode)

  if(blocklines(1).le.0)then
    write(*,*)'Warning: was not able to read the total decay width of particle with PDG code:',particlePDGcode
  else

    i=blocklines(1)
    temp_line=edited_contents_of_input_file(i)%line

    stringlength =len(temp_line)
    stringlength2=len(col) 

    call split_into_col(temp_line,stringlength,col,stringlength2)

    read(col(2),*)particlePDGcode_fromblock
    if(particlePDGcode.ne.particlePDGcode_fromblock)then
      stop'problem in function get_particledecaywidth(2)'
    endif
  
    read(col(3),*)get_totaldecaywidth
  endif

  deallocate(col)

 end function get_totaldecaywidth
 !********************************************************
 function get_SPhenocrosssectionCMenergy(particlePDGcode_1,particlePDGcode_2,pol_1,pol_2,incISR)
 !get the centre of mass of the SPheno cross section
 !********************************************************
  implicit none
  integer,intent(in) :: particlePDGcode_1,particlePDGcode_2,incISR
  double precision,intent(in) :: pol_1,pol_2
  integer :: incISR_fromblock
  double precision :: pol_1_fromblock,pol_2_fromblock
  integer :: blocklines(2),particlePDGcode_req(2),particlePDGcode_fromblock(2)
  double precision :: get_SPhenocrosssectionCMenergy
  character(len=50),allocatable :: col(:)
  character(len=200) :: temp_line
  integer :: stringlength,stringlength2
  integer :: i

  allocate(col(7))
  get_SPhenocrosssectionCMenergy=0.0D0
  particlePDGcode_req(1)=particlePDGcode_1
  particlePDGcode_req(2)=particlePDGcode_2

  blocklines=line_numbers_of_block('SPhenoCrossSections')

  if(minval(blocklines).le.0)then
    write(*,*)'Warning: was not able to read the block SPhenoCrossSections'
 
  else
    i=blocklines(1)+1
    temp_line=edited_contents_of_input_file(i)%line

    stringlength =len(temp_line)
    stringlength2=len(col) 

    call split_into_col(temp_line,stringlength,col,stringlength2)

    read(col(2),*)particlePDGcode_fromblock(1)
    read(col(3),*)particlePDGcode_fromblock(2)
    read(col(5),*)pol_1_fromblock
    read(col(6),*)pol_2_fromblock 
    read(col(7),*)incISR_fromblock

    if(trim(col(1))           .ne. 'XS'                       )then
      stop'problem in function get_SPhenocrosssectionCMenergy(1)'
    elseif( .not.same_particles(particlePDGcode_req,particlePDGcode_fromblock) )then
      stop'problem in function get_SPhenocrosssectionCMenergy(2)'
    elseif( pol_1             .ne. pol_1_fromblock            )then
      stop'problem in function get_SPhenocrosssectionCMenergy(4)'
    elseif( pol_2             .ne. pol_2_fromblock            )then
      stop'problem in function get_SPhenocrosssectionCMenergy(5)'
    elseif( incISR_fromblock  .ne. incISR_fromblock           )then
      stop'problem in function get_SPhenocrosssectionCMenergy(6)'
    endif

    read(col(4),*)get_SPhenocrosssectionCMenergy
  endif

  deallocate(col)

 end function get_SPhenocrosssectionCMenergy
 !********************************************************
 function get_twobodybranchingratio(particlePDGcode_parent,particlePDGcode_da1,particlePDGcode_da2)
 !********************************************************
  implicit none
  integer,intent(in) :: particlePDGcode_parent,particlePDGcode_da1,particlePDGcode_da2
  integer :: particlePDGcode_fromblock_da(2),particlePDGcode_req(2)
  integer :: blocklines(2)
  double precision :: get_twobodybranchingratio
  character(len=50),allocatable :: col(:)
  character(len=200) :: temp_line
  integer :: stringlength,stringlength2
  integer :: i
  integer :: nda

  allocate(col(4))
  get_twobodybranchingratio=0.0D0
  blocklines=line_numbers_of_decay(particlePDGcode_parent)
  particlePDGcode_req(1)=particlePDGcode_da1
  particlePDGcode_req(2)=particlePDGcode_da2

  if(minval(blocklines).le.0)then
    write(*,*)'Warning: was not able to read the decay widths of particle with PDG code',particlePDGcode_parent
  else

    do i=1+blocklines(1),blocklines(2)
     temp_line=edited_contents_of_input_file(i)%line

     if(trim(temp_line).ne.'')then

       stringlength =len(temp_line)
       stringlength2=len(col) 

       call split_into_col(temp_line,stringlength,col,stringlength2)

       read(col(2),*)nda
       if(nda.eq.2)then
        read(col(3),*)particlePDGcode_fromblock_da(1)
        read(col(4),*)particlePDGcode_fromblock_da(2)

        if(same_particles(particlePDGcode_req,particlePDGcode_fromblock_da))then

          read(col(1),*)get_twobodybranchingratio
    
        endif
       endif
     endif
    enddo
  endif

  deallocate(col)

 end function get_twobodybranchingratio
 !********************************************************
 function get_threebodybranchingratio(particlePDGcode_parent,particlePDGcode_da1,particlePDGcode_da2,particlePDGcode_da3)
 !********************************************************
  implicit none
  integer,intent(in) :: particlePDGcode_parent,particlePDGcode_da1,particlePDGcode_da2,particlePDGcode_da3
  integer :: particlePDGcode_fromblock_da(3),particlePDGcode_req(3)
  integer :: blocklines(2)
  double precision :: get_threebodybranchingratio
  character(len=50),allocatable :: col(:)
  character(len=200) :: temp_line
  integer :: stringlength,stringlength2
  integer :: i
  integer :: nda

  allocate(col(5))
  get_threebodybranchingratio=0.0D0
  blocklines=line_numbers_of_decay(particlePDGcode_parent)
  particlePDGcode_req(1)=particlePDGcode_da1
  particlePDGcode_req(2)=particlePDGcode_da2
  particlePDGcode_req(3)=particlePDGcode_da3

  if(minval(blocklines).le.0)then
    write(*,*)'Warning: was not able to read the decay widths of particle with PDG code',particlePDGcode_parent
  else

    do i=1+blocklines(1),blocklines(2)
     temp_line=edited_contents_of_input_file(i)%line

     if(trim(temp_line).ne.'')then

       stringlength =len(temp_line)
       stringlength2=len(col) 

       call split_into_col(temp_line,stringlength,col,stringlength2)

       read(col(2),*)nda
       if(nda.eq.3)then
        read(col(3),*)particlePDGcode_fromblock_da(1)
        read(col(4),*)particlePDGcode_fromblock_da(2)
        read(col(5),*)particlePDGcode_fromblock_da(3)

        if(same_particles(particlePDGcode_req,particlePDGcode_fromblock_da))then

          read(col(1),*)get_threebodybranchingratio
    
        endif
       endif
     endif
    enddo
  endif

  deallocate(col)

 end function get_threebodybranchingratio

 !********************************************************
 function get_SPhenocrosssection(particlePDGcode_da1,particlePDGcode_da2)
 !********************************************************
  implicit none
  integer,intent(in) :: particlePDGcode_da1,particlePDGcode_da2
  integer :: particlePDGcode_fromblock_da(2),particlePDGcode_req(2)
  integer :: blocklines(2)
  double precision :: get_SPhenocrosssection
  character(len=50),allocatable :: col(:)
  character(len=200) :: temp_line
  integer :: stringlength,stringlength2
  integer :: i
  integer :: nda

  allocate(col(4))
  get_SPhenocrosssection=0.0D0
  blocklines=line_numbers_of_block('SPhenoCrossSections')
  particlePDGcode_req(1)=particlePDGcode_da1
  particlePDGcode_req(2)=particlePDGcode_da2

  if(minval(blocklines).le.0)then
    write(*,*)'Warning: was not able to read the block SPhenoCrossSections'
  else

    do i=1+blocklines(1),blocklines(2)
     temp_line=edited_contents_of_input_file(i)%line

     if(trim(temp_line).ne.'')then

       stringlength =len(temp_line)
       stringlength2=len(col) 

       call split_into_col(temp_line,stringlength,col,stringlength2)

       read(col(2),*)nda
       if(nda.eq.2)then
         read(col(3),*)particlePDGcode_fromblock_da(1)
         read(col(4),*)particlePDGcode_fromblock_da(2)

         if(same_particles(particlePDGcode_req,particlePDGcode_fromblock_da))then

           read(col(1),*)get_SPhenocrosssection
    
         endif
       endif
     endif
    enddo
  endif

  deallocate(col)

 end function get_SPhenocrosssection

 !********************************************************
 function get_HiggsCouplingsBosons(particlePDGcode_1,particlePDGcode_2,particlePDGcode_3)
 !********************************************************
 !doesn't matter if particles or antiparticles are used: takes abs() of the PDG number anyway
  implicit none
  integer,intent(in) :: particlePDGcode_1,particlePDGcode_2,particlePDGcode_3
  integer :: particlePDGcode_req(3)
  integer :: particlePDGcode_fromblock(3)
  integer :: blocklines(2)
  double precision :: get_HiggsCouplingsBosons
  character(len=50),allocatable :: col(:)
  character(len=200) :: temp_line
  integer :: stringlength,stringlength2
  integer :: i
  integer :: np

  allocate(col(5))
  get_HiggsCouplingsBosons=0.0D0
  blocklines=line_numbers_of_block('HiggsBoundsInputHiggsCouplingsBosons')
  particlePDGcode_req(1)=particlePDGcode_1
  particlePDGcode_req(2)=particlePDGcode_2
  particlePDGcode_req(3)=particlePDGcode_3
  particlePDGcode_req=abs(particlePDGcode_req)

  if(minval(blocklines).le.0)then
   write(*,*)'Problem reading block HiggsBoundsInputHiggsCouplingsBosons:'
   write(*,*)'Note that this is a non-standard SLHA block and may'
   write(*,*)'need to be entered into the SLHA input file by hand.'
  else

    do i=1+blocklines(1),blocklines(2)
     temp_line=edited_contents_of_input_file(i)%line

     if(trim(temp_line).ne.'')then

       stringlength =len(temp_line)
       stringlength2=len(col) 

       call split_into_col(temp_line,stringlength,col,stringlength2)
  
       read(col(2),*)np
       if(np.eq.3)then
        read(col(3),*)particlePDGcode_fromblock(1)
        read(col(4),*)particlePDGcode_fromblock(2)
        read(col(5),*)particlePDGcode_fromblock(3)
        particlePDGcode_fromblock=abs(particlePDGcode_fromblock)

        if(same_particles(particlePDGcode_req,particlePDGcode_fromblock))then

          read(col(1),*)get_HiggsCouplingsBosons
    
        endif
       endif
     endif
    enddo
  endif


  deallocate(col)

 end function get_HiggsCouplingsBosons

 !********************************************************
 function get_HiggsCouplingsFermions(particlePDGcode_1,particlePDGcode_2,particlePDGcode_3)
 !********************************************************
 !doesn't matter if particles or antiparticles are used: takes abs() of the PDG number anyway
  implicit none
  integer,intent(in) :: particlePDGcode_1,particlePDGcode_2,particlePDGcode_3
  integer :: particlePDGcode_req(3)
  integer :: particlePDGcode_fromblock(3)
  integer :: blocklines(2)
  double precision :: get_HiggsCouplingsFermions(2)
  character(len=50),allocatable :: col(:)
  character(len=200) :: temp_line
  integer :: stringlength,stringlength2
  integer :: i
  integer :: np

  allocate(col(6))
  get_HiggsCouplingsFermions=0.0D0
  blocklines=line_numbers_of_block('HiggsBoundsInputHiggsCouplingsFermions')
  particlePDGcode_req(1)=particlePDGcode_1
  particlePDGcode_req(2)=particlePDGcode_2
  particlePDGcode_req(3)=particlePDGcode_3
  particlePDGcode_req=abs(particlePDGcode_req)

  if(minval(blocklines).le.0)then
   write(*,*)'Problem reading block HiggsBoundsInputHiggsCouplingsFermions:'
   write(*,*)'Note that this is a non-standard SLHA block and may'
   write(*,*)'need to be entered into the SLHA input file by hand.'
  else

    do i=1+blocklines(1),blocklines(2)
     temp_line=edited_contents_of_input_file(i)%line

     if(trim(temp_line).ne.'')then

       stringlength =len(temp_line)
       stringlength2=len(col) 

       call split_into_col(temp_line,stringlength,col,stringlength2)

       read(col(3),*)np
       if(np.eq.3)then
        read(col(4),*)particlePDGcode_fromblock(1)
        read(col(5),*)particlePDGcode_fromblock(2)
        read(col(6),*)particlePDGcode_fromblock(3)
        particlePDGcode_fromblock=abs(particlePDGcode_fromblock)

        if(same_particles(particlePDGcode_req,particlePDGcode_fromblock))then

          read(col(1),*)get_HiggsCouplingsFermions(1)
          read(col(2),*)get_HiggsCouplingsFermions(2)    
        endif
       endif
     endif
    enddo
  endif


  deallocate(col)

 end function get_HiggsCouplingsFermions
 !********************************************************
 function get_mass(particlePDGcode)
 !********************************************************
  implicit none
  integer,intent(in) :: particlePDGcode
  integer :: particlePDGcode_fromblock
  integer :: blocklines(2)
  integer :: i
  integer :: stringlength,stringlength2
  character(len=200) :: temp_line
  double precision :: get_mass
  character(len=50),allocatable :: col(:)

  allocate(col(2))
  get_mass=-1.0D0

  blocklines=line_numbers_of_block('mass')
  if(minval(blocklines).le.0)then
   write(*,*)'Problem reading the block MASS'
   write(*,*)'This block is necessary input to HiggsBounds.'  
  endif

  do i=1+blocklines(1),blocklines(2)
   temp_line=edited_contents_of_input_file(i)%line

   if(trim(temp_line).ne.'')then

     stringlength =len(temp_line)
     stringlength2=len(col) 

     call split_into_col(temp_line,stringlength,col,stringlength2)

     read(col(1),*)particlePDGcode_fromblock

     if(particlePDGcode.eq.particlePDGcode_fromblock)then
       read(col(2),*)get_mass
    endif
   endif
  enddo

  deallocate(col)

 end function get_mass
 !********************************************************
 function get_modsel(modsel_id)
 !********************************************************
  implicit none
  integer,intent(in) :: modsel_id
  integer :: modsel_id_fromblock
  integer :: blocklines(2)
  integer :: i
  integer :: stringlength,stringlength2
  character(len=200) :: temp_line
  integer :: get_modsel
  character(len=50),allocatable :: col(:)

  allocate(col(2))
  get_modsel=0

  blocklines=line_numbers_of_block('modsel')
  if(minval(blocklines).le.0)then
   write(*,*)'Problem reading the block modsel'
  else

    do i=1+blocklines(1),blocklines(2)
     temp_line=edited_contents_of_input_file(i)%line

     if(trim(temp_line).ne.'')then

      stringlength =len(temp_line)
      stringlength2=len(col) 

      call split_into_col(temp_line,stringlength,col,stringlength2)

      read(col(1),*)modsel_id_fromblock

      if(modsel_id.eq.modsel_id_fromblock)then
       read(col(2),*)get_modsel
      endif
     endif
    enddo
  endif

  deallocate(col)

 end function get_modsel 
!********************************************************
 function same_particles(array1,array2)
 !********************************************************
  implicit none
  integer,intent(in) :: array1(:)
  integer,intent(in) :: array2(:)

  integer,allocatable :: array1_reordered(:)
  integer,allocatable :: array2_reordered(:)

  integer :: i
  logical :: same_particles

  if(lbound(array1,dim=1).ne.lbound(array2,dim=1))then
   stop'problem in function same_particles'
  endif

  if(ubound(array1,dim=1).ne.ubound(array2,dim=1))then
   stop'problem in function same_particles'
  endif

  allocate(array1_reordered(lbound(array1,dim=1):ubound(array1,dim=1)))
  allocate(array2_reordered(lbound(array1,dim=1):ubound(array1,dim=1)))

  call reorderPDG(array1,array1_reordered)
  call reorderPDG(array2,array2_reordered)

  same_particles=.True.
  do i=lbound(array1_reordered,dim=1),ubound(array1_reordered,dim=1)
   if(array1_reordered(i).ne.array2_reordered(i))same_particles=.False.
  enddo

  deallocate(array1_reordered)
  deallocate(array2_reordered)

 end function same_particles

 !********************************************************
 subroutine reorderPDG(array_in,array_out)
 !********************************************************
  implicit none
  integer,intent(in) :: array_in(:)
  integer,allocatable :: array_in_temp(:)
  integer,intent(out) :: array_out(:)
  integer,parameter :: veryhighnumber=1000000000 !should be higher than all the PDG numbers
  integer :: i, minl

  allocate(array_in_temp(lbound(array_in,dim=1):ubound(array_in,dim=1)))

  array_in_temp=array_in
  array_out=0

  do i=lbound(array_in_temp,dim=1),ubound(array_in_temp,dim=1)
    minl=minloc(array_in_temp,dim=1)
    array_out(i)=array_in_temp(minl)
    array_in_temp(minl)=veryhighnumber
  enddo
   
  deallocate(array_in_temp)

 end subroutine reorderPDG
 !********************************************************
 function line_numbers_of_block(name_of_block)
 !********************************************************
  character(len=*),intent(in) ::name_of_block
  integer :: line_numbers_of_block(2)

  line_numbers_of_block=line_numbers_of_block_or_decay('block',name_of_block)
 end function line_numbers_of_block

 !********************************************************
 function line_numbers_of_decay(particlePDGcode)
 !********************************************************
  character(len=7) ::particlePDGcode_string
  integer :: line_numbers_of_decay(2)
  integer,intent(in) :: particlePDGcode

  write(particlePDGcode_string,'(I7)')particlePDGcode

  line_numbers_of_decay=line_numbers_of_block_or_decay('decay',particlePDGcode_string)

 end function line_numbers_of_decay

 !********************************************************
 function line_numbers_of_block_or_decay(typ,name_of_block)
 !********************************************************
  integer :: x,i
  integer :: line_numbers_of_block_or_decay(2)
  logical :: block_has_started
  character(len=*),intent(in) ::name_of_block
  character(len=50) ::name_of_block_lowercase
  character(len=200) :: temp_line
  character(len=5),intent(in) :: typ
  integer :: stringlength,stringlength2
  character(len=50),allocatable :: col(:)

  allocate(col(2))

  select case(typ)
  case('block')
  case('decay')
  case default
   stop'wrong input to function line_numbers_of_block_or_decay'
  end select

  if(len(name_of_block).gt.len(name_of_block_lowercase))stop'error in function line_numbers_of_block_or_decay (a)'

  block_has_started=.False.

  stringlength=len(name_of_block)
  name_of_block_lowercase=adjustl(strtolcase(name_of_block,stringlength))

  line_numbers_of_block_or_decay=0

  do x=1,ubound(edited_contents_of_input_file,dim=1)


    temp_line=edited_contents_of_input_file(x)%line
    stringlength =len(temp_line)
    stringlength2=len(col) 

    call split_into_col(temp_line,stringlength,col,stringlength2)

    stringlength2=len(col) 
    do i=1,ubound(col,dim=1)
     col(i) = strtolcase( col(i) ,stringlength2)
    enddo
 
    if(block_has_started)then
     if((col(1).eq.'block').or.(col(1).eq.'decay'))then
      exit
     endif
     line_numbers_of_block_or_decay(2)=x
    endif
   if(col(1).eq.typ)then

    if(trim(col(2)).eq.trim(name_of_block_lowercase))then
      block_has_started=.True.
      line_numbers_of_block_or_decay(1)=x
    endif
   endif

  enddo

  deallocate(col)
 end function line_numbers_of_block_or_decay
                                                                                                              
 !****************************************************
 function getSLHAfilelength(fileid)
 !****************************************************
 ! calculates file length and checks for errors
 !**************************************************** 
  implicit none
  !--------------------------------------input 
  integer fileid   
  !-----------------------------------function 
  integer :: getSLHAfilelength
  !-----------------------------------internal      
  integer :: n,ios
  character(LEN=5) :: filechar
  character(LEN=20) :: sample
  !-------------------------------------------      
  
  write(filechar,'(I5)')fileid      

  !this will count the number of lines in the file, including the last one
  n = 0                        
  do
   read(fileid,'(a)',iostat=ios) sample

   if(ios.lt.0)then
    exit
   elseif(ios.gt.0) then
    stop 'error in SLHA input file'
   endif
   n = n + 1               
  enddo            

  if(n.eq.0)stop'File is empty'
      
  getSLHAfilelength=n
  rewind(fileid)

 end function getSLHAfilelength

 !****************************************************
end module SLHA_manip
!********************************************************
